/*
 * PIC32 peripheral access utilities.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ASSEMBLER__

#ifndef __PIC32_C_H__
#define __PIC32_C_H__

#include <stdint.h>
#include "mips.h"
#include "pic32.h"



/* Access. */

#define REG(mem)                *((volatile uint32_t *) (mem))



/* Bit clearing, setting and inverting. */

static inline void CLR_REG(uint32_t mem, uint32_t val)
{
    REG(mem + CLR) = val;
}

static inline void SET_REG(uint32_t mem, uint32_t val)
{
    REG(mem + SET) = val;
}

static inline void INV_REG(uint32_t mem, uint32_t val)
{
    REG(mem + INV) = val;
}



/* Address translation. */

static inline uint32_t PHYSICAL(uint32_t addr)
{
    return ((uint32_t) addr) - KSEG0_BASE;
}

static inline uint32_t HW_PHYSICAL(uint32_t addr)
{
    return ((uint32_t) addr) - KSEG1_BASE;
}



/* Register collection access. */

static inline uint32_t DMA_REG(int channel, uint32_t reg)
{
    return DCHBASE + reg + (channel - DCHMIN) * DCHSTEP;
}

static inline uint32_t OC_REG(int unit, uint32_t reg)
{
    return OCBASE + reg + (unit - OCMIN) * OCSTEP;
}

static inline uint32_t PM_REG(int port, uint32_t reg)
{
    return PMBASE + reg + (port - PMMIN) * PMSTEP;
}

static inline uint32_t TIMER_REG(int timer, uint32_t reg)
{
    return TIMERBASE + reg + (timer - TIMERMIN) * TIMERSTEP;
}

static inline uint32_t UART_REG(int uart, uint32_t reg)
{
    return UARTBASE + reg + (uart - UARTMIN) * UARTSTEP;
}



/* Convenience types. */

enum dma_chain
{
    dma_chain_none,
    dma_chain_next,
    dma_chain_previous,
};

#endif /* __PIC32_C_H__ */

#endif /* __ASSEMBLER__ */
